/****** ETroepfel.c ***********************************************
*
*   PROGRAM
*      ETroepfel
*
*   CONTENTS
*      A function that calculates e step by step an sends result to
*      a stream. If the stream is not given, its send to stdout.
*
*   AUTHOR
*      Georg Pfundt
*
*   COPYRIGHT
*       by Georg Pfundt 03-JAN-1996  18:59:36
*
*   LANGUAGE
*      ANSI-C
*
*   TRANSLATOR
*      SAS-C V6.55
*
*   HISTORY
*      1.0, GeP, 03-JAN-1996, first version
*      <Version, Autor, Datum, Bemerkung>
*
*   SUPPORT
*      Literature:
*
*      Spektrum der Wissenschaft, Stewart, Ian; n12 a1995 pp10-14
*
*      "A Spigot Algorithm for the Digits of Pi", Rabinowitz, Stanley;
*      Wagon, Stan; American Mathematical Monthly, v102 n3 a1995 pp195-203
*
*      "The Calculation of e to many Significant Digits", Sale, A. H. J.;
*      Computing Journal, v11 a1968 pp229-230
*
*   IMPORTS
*      None.
*
*   BUGS
*      None known.
*
*   ADDRESS
*      Georg Pfundt, Im Oberviertel 18, 76229 Karlsruhe, Germany
*
*   PHONE
*      GERMANY - (0)721/481591
*      E-Mail: PFUNDT@PHYSIK.UNI-KARLSRUHE.DE
*          or: gp@ict.fhg.de  (till 30-APR-1996)
*
*   UPDATE
*
*   USAGE
*      void ETroepfel(long sig, FILE *out);
*
*      sig - number of significant places to compute.
*      out - a C-filehandle or NULL for stdout.
*
********************************************************************
* TABSIZE=2
*/

#include <stdio.h>
#include <stdlib.h>
 /* #include <string.h> */



/* memory allocation function for amiga because of limitations with malloc
 AND shortints */
#ifdef _SHORTINT
#include <proto/exec.h>
#define ALLOCMEM(x) AllocVec((ULONG)(x),NULL)
#define FREEMEM(x) FreeVec(x)
#else
/* for other(=unix) systems */
#define ALLOCMEM(x) malloc((size_t)(x))
#define FREEMEM(x) free(x)
#endif

/* check for amiga */
#ifndef _AMIGA
#define __aligned
#endif

/* type of digits*/
typedef unsigned long digit_t;


__aligned long
ETroepfel(long sig,FILE *out)
{
	long ndig,h,d,i,j;
	digit_t *dig,*ptr;

	/* set output to stdout if not given */
	if (out == NULL)
		out=stdout;

	/* number of digits we need*/
	ndig = sig+2;

	/* alloc mem for digits */
	if ((dig = (digit_t*)ALLOCMEM(ndig*sizeof(digit_t))) == NULL)
	{
		printf ("***Error: No memory (%ld Bytes)!\n\n",ndig*sizeof(digit_t));
		return 0;
	}

	/* fill mem with 1 */
	i=ndig;
	ptr=dig;
	while(i--)
		*(ptr++)=1;

	/* output first digit */
	putc ('2',out);
	putc ('.',out);

	/* start algo */
	for (i=3 ; i<ndig ; i++)
	{
		ptr=dig+ndig-1;
		if (*ptr > 6553)
		{
			printf ("***Error: Overflow1!\n");
			break;
		}

		/* multiply first digit by 10 */
		*ptr *= 10;

		j=ndig;
		while(--j > 0)
		{
			h=(long)(*ptr);
			d=j+2;
      *ptr = h % d;
      h = (*(--ptr))*10 + (h / d);
      if (h > 65535)
			{
				printf ("\n***Error: Overflow2 at i=%ld, j=%ld, r=%lu,"
								"d=%ld\n",i,j,(unsigned long)(*ptr),d);
				break;
			}
      *ptr = h;
		}
		/* overflow exit */
		if (h>65535)
			break;

		/* new digit is last carry */
		h = *dig;
		*dig = h%2;
		putc ((char)('0'+ h/2),out);

		/* flush every 10 digits */
		if ((i % 10) == 0)
			fflush(out);
	}

	/* end with NL */
	putc ('\n',out);
	fflush(out);

	/* free memory */
	FREEMEM(dig);
}
