#if 0
  ## This makes it selfcompiling under unix
  ## just type:
  ## sh Troepfel.c
  echo 'gcc Troepfel.c PiTroepfel.c ETroepfel.c -o troepfel -O2'
  exec gcc Troepfel.c PiTroepfel.c ETroepfel.c -o troepfel -O2
  exit
#endif

/****** Troepfel.c ***********************************************
*
*   PROGRAM
*      Troepfel
*
*   CONTENTS
*      Call the two "troepfel"-Algorithms for pi and e (its just the user
*      interface).
*
*   AUTHOR
*      Georg Pfundt
*
*   COPYRIGHT
*       by Georg Pfundt 03-JAN-1996  11:33:08
*
*   LANGUAGE
*      ANSI-C
*
*   TRANSLATOR
*      SAS-C V6.55
*
*   HISTORY
*      1.0, GeP, 03-JAN-1996, first version
*      <Version, Autor, Datum, Bemerkung>
*
*   SUPPORT
*      Literature.
*
*   IMPORTS
*      None.
*
*   BUGS
*      None known.
*
*   ADDRESS
*      Georg Pfundt, Im Oberviertel 18, 76229 Karlsruhe, Germany
*
*   PHONE
*      GERMANY - (0)721/481591
*      E-Mail: PFUNDT@PHYSIK.UNI-KARLSRUHE.DE
*          or: gp@ict.fhg.de  (till 30-APR-1996)
*
*   UPDATE
*
*   USAGE
*      Usage.
*
********************************************************************
*
*/

#include <stdio.h>
#include <stdlib.h>

#include "Troepfel.h"
#include <time.h>
#include <ctype.h>


/* unix does not now about clocks per sec */
#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC 1000L
#define CLOCKUNIT 1000L
#else
#define CLOCKUNIT 1L
#endif


#define VERSION "1.0"

/* define version string for amiga VERSION command*/
#ifdef _AMIGA
const char *version="$VER: Troepfel V"VERSION"   by Georg Pfundt "__AMIGADATE__;
#endif

const char *hello="Trpfel Algorithm V"VERSION"  by Georg Pfundt 03-JAN-1996\n\
Calculates Pi or e to many many digits!\n\
\nUsage: %s [PI|E] <numdigits> [<outputfile>]%s";

int
main(int argc, char *argv[])
{
	long dig,sec;
	clock_t start,stop;
	FILE *fp;

	/* check for WorkBench Startup on amiga */
#ifdef _AMIGA
	/* no wb-startup! */
	if (argc==0)
	{
		printf (hello,"Troepfel","\nPlease use from CLI!");
		return 20;
	}
#endif

	if ((argc < 2) || (argc > 4))
	{
		printf (hello,argv[0],"\n\n");
		return 5;
	}
	dig = atol(argv[2]);


	if (argc == 4)
	{
		/* open output file */
		if ((fp = fopen(argv[3],"w")) == NULL)
		{
			printf ("***Error: could not open outputfile '%s'!\n",argv[3]);
			return 20;
		}
	}
	else
		fp=NULL;


	/* check time */
	start = clock()/CLOCKUNIT;

	/* call calculation for pi or e */
	switch (tolower(*argv[1]))
	{
	case 'p':
		dig = PiTroepfel(dig,fp);
		break;
	case 'e':
		ETroepfel(dig,fp);
		break;
	default:
		printf ("Sorry, I can only calculate Pi and e, but not %s!\n",argv[1]);
		return 20;
	}

	stop = (clock()/CLOCKUNIT)+1L;

	sec=(stop-start)/CLOCKS_PER_SEC;

	printf ("\n %ld significant digits of %s Calculated in %ld.%02ld sec.\n"
					"Digits per sec.: %ld.%02ld \n",dig,argv[1],
				sec,(((stop-start)*100)/CLOCKS_PER_SEC) % 100,
				(CLOCKS_PER_SEC*dig)/(stop-start),
				((CLOCKS_PER_SEC*dig*100)/(stop-start)) % 100);

	return 0;
}
